import json
from .base import HomeServerApp


class Transmission(HomeServerApp):
    def setup(self):
        if self.container:
            self.container.stop()
        cfg = json.loads((self.path / 'config' / 'settings.json').read_bytes())
        cfg['rpc-whitelist-enabled'] = False
        (self.path / 'config' / 'settings.json').write_text(json.dumps(cfg))
        self.compose.start()



import docker
# from bacchus.homeassistant import HomeAssistant
from bacchus.jackett import Jackett
from bacchus.transmission import Transmission
from bacchus.lidarr import Lidarr
from bacchus.radarr import Radarr
from bacchus.medusa import Medusa
from bacchus.nextcloud import NextCloud
from bacchus.nginx import Nginx
from bacchus.compose import DockerCompose
from bacchus.openvpn import OpenVPN
from bacchus.jellyfin import Jellyfin
from bacchus.certificates import CertManager

__all__ = [
    CertManager, Nginx, OpenVPN, NextCloud, Transmission, Jackett, Lidarr,
    Radarr, Medusa, Jellyfin
]


class HomeServerSetup:
    """Base cmdline class.

    Not currently extending a CLEO App so this can be extended easily.
    Params are the same as BaseHomeApp class from `base` module.
    """
    def __init__(self, domain, docker_prefix="bacchus", **kwargs):
        """Setup providers.

        Kwargs will be inherited as metadata, for example, nextcloud will use the nextcloud_username
        and nextcloud_password variables
        """
        client = docker.from_env()
        self.providers = {}
        self.compose = DockerCompose(domain, client, docker_prefix, None, self,
                                     **kwargs)
        self.providers.update({
            cls.__name__: cls(domain, client, docker_prefix, self.compose,
                              self, **kwargs)
            for cls in __all__
        })

    def configure(self, provider_name=None):
        """Configure given providers."""
        if provider_name:
            return self.providers[provider_name].setup()

        self.compose.copy_template()
        self.compose.create_env_files()
        self.compose.start()

        for provider in self.providers.values():
            provider.wait_for_status()
            provider.wait_for_config()
            provider.setup()
        self.compose.restart()



from cleo import Command
from cleo import Application

from . import HomeServerSetup


class GreetCommand(Command):
    """
    Installs bacchus.
    Requires:
        - Valid DNS hostname registered in a supported provider
        - NAT or direct exposure of the given machine's openvpn port (1194)

    install
        {dns-domain? : Domain (FQDN) for virtualhosts}
        {username? : Nextcloud first user's username}
        {password? : Nextcloud first user's password}
        {dns-key? : DNS Provider (ghandi) API key}
        {iface? : (Optional) Main network interface name}
        {service? : (Optional) Set up only one service}
    """
    def handle(self):
        """Handle command"""
        setup = HomeServerSetup(domain=self.argument('domain'),
                                nextcloud_username=self.argument('username'),
                                nextcloud_password=self.argument('password'),
                                iface=self.argument('iface'),
                                dns_api_key=self.argument('dns'))

        setup.configure(self.argument('service'))


def main():
    application = Application()
    application.add(GreetCommand())
    application.run()



from pathlib import Path
from time import sleep
import docker
import logging

TEMPLATES = Path(__file__).parent / 'templates'
DOCKER_PATH = Path(__file__).parent.parent.parent / 'docker'

logging.basicConfig(level=logging.INFO)


class HomeServerApp:
    """Base APP Class.

    All common actions (config file placement, docker controls...) go here
    """
    def __init__(self, domain, client, docker_prefix, compose, parent, **kwargs):
        name = self.__class__.__name__.lower()
        self.providers = parent.providers
        self.compose = compose
        self.path = DOCKER_PATH / 'data' / name
        self.domain = domain
        self.meta = kwargs
        self.meta['project_name'] = docker_prefix
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel(logging.DEBUG)
        self.path.mkdir(parents=True, exist_ok=True)
        self.client = client
        self.prefix = docker_prefix

    @property
    def containers(self):
        """Return a list of all containers associated with current docker"""
        return {
            a.name.split('_')[1]: a
            for a in self.client.containers.list()
            if a.name.split('_')[0] == self.prefix
        }

    @property
    def container(self):
        return self.containers.get(self.__class__.__name__.lower())

    @property
    def running(self):
        if not self.container:
            return False
        return self.container.status == 'running'

    def wait_for_status(self):
        """Wait for container to start."""
        while not self.running:
            self.logger.debug(
                f'Waiting for {self.__class__.__name__} to start')
            sleep(1)

    def wait_for_config(self):
        """If we are waiting for a configuration file to be written, wait for it."""
        self.logger.debug(f'Waiting for {self.__class__.__name__} config')
        if hasattr(self, "config_file"):
            while not self.config_file.exists():
                sleep(1)



import configparser
import json
from .base import TEMPLATES
from .base import HomeServerApp


class Medusa(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config.ini'

    def setup_nginx(self):
        config = configparser.ConfigParser()
        config.read(str(self.config_file))
        config.set('General', 'web_root', '/tv')
        with open(self.config_file, 'w') as fileo:
            config.write(fileo)

    def setup_indexers(self):
        config = configparser.ConfigParser()
        config.read(str(self.config_file))
        api_key = json.loads((self.path / '..' / 'jackett' / 'Jackett' /
                              'ServerConfig.json').read_text())['APIKey']

        providers = []
        for prov in (TEMPLATES / 'jackett' / 'Indexers').glob('*.json'):
            providers.append(prov.stem.lower())
            local = dict(
                name=prov.stem,
                cat_ids=('5010, 5030, 5040, 2000, 2010, '
                         '2020, 2030, 2035, 2040, 2045, 2050, 2060'),
                api_key=api_key,
                search_mode='eponly',
                search_fallback=0,
                enable_daily=1,
                enable_backlog=1,
                enable_manualsearch=1,
                enable_search_delay=0,
                ratio='',
                search_delay=480,
                minseed=1,
                url=(f'https://private.{self.domain}/trackers/api/v2.0/'
                     f'indexers/{prov.stem}/results/torznab/'))
            if not config.has_section(prov.stem.upper()):
                config.add_section(prov.stem.upper())
            config[prov.stem.upper()].update({
                f'{prov.stem}_{prop}': str(val)
                for prop, val in local.items()
            })
            config[prov.stem.upper()][prov.stem.lower()] = '1'

            if not config.has_section('TORRENT'):
                config.add_section('TORRENT')

            config['General']['process_method'] = 'move'
            config['General']['process_automatically'] = '1'
            config['General']['tv_download_dir'] = '/downloads'

            config['TORRENT'].update(
                dict(torrent_username='',
                     torrent_password='',
                     torrent_host=f'https://private.{self.domain}',
                     torrent_path='',
                     torrent_seed_time='3',
                     torrent_paused='0',
                     torrent_high_bandwidth='0',
                     torrent_label='',
                     torrent_label_anime='',
                     torrent_verify_cert='0',
                     torrent_rpcurl='transmission',
                     torrent_auth_type='',
                     torrent_seed_location=''))

        config['Torznab']['torznab_providers'] = ','.join(providers)

        with open(self.config_file, 'w') as fileo:
            config.write(fileo)

    def setup(self):
        if self.container:
            self.container.stop()
        self.setup_nginx()
        self.setup_indexers()
        self.compose.start()



import json
from functools import lru_cache
from .base import TEMPLATES
import sqlite3
import xml.etree.ElementTree as ET

from .base import HomeServerApp


class Lidarr(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config.xml'

    @property
    @lru_cache()
    def config(self):
        return ET.parse(str(self.config_file))

    def setup_nginx(self):
        self.config.find('UrlBase').text = '/music/'
        self.config.write(str(self.config_file))

    def setup_indexers(self):
        api_key = json.loads((self.path / '..' / 'jackett' / 'Jackett' /
                              'ServerConfig.json').read_text())['APIKey']
        indexer_files = (TEMPLATES / 'jackett' / 'Indexers').glob('*.json')
        indexers = []
        for num, name in enumerate((a.stem.lower() for a in indexer_files)):
            base_url = (f"https://private.{self.domain}/trackers/api/v2.0/"
                        f"indexers/{name}/results/torznab/")
            categories = [3000, 3010, 3020, 3030, 3040]
            settings = {
                "minimumSeeders": 1,
                "baseUrl": base_url,
                "apiKey": api_key,
                "apiPath": "/api",
                "seedCriteria": "{}",
                "categories": categories,
            }
            indexers.append([
                num + 1, name, 'Torznab',
                json.dumps(settings), 'TorznabSettings', 1, 1, 1
            ])

        conn = sqlite3.connect(str((self.path / 'nzbdrone.db').absolute()))
        cursor = conn.cursor()
        cursor.executemany('insert into Indexers values (?, ?, ?, ?, ?, ?, ?)',
                           indexers)
        conn.commit()
        conn.close()



    def setup(self):
        self.setup_nginx()



from contextlib import suppress
import json
import shutil
from .base import HomeServerApp
from .base import TEMPLATES


class Jackett(HomeServerApp):
    def setup(self):
        self.container.stop()
        self.setup_nginx()
        self.compose.start()
        self.copy_indexers()

    def copy_indexers(self):
        indexers_path = (self.path / 'Jackett' / 'Indexers')
        with suppress(Exception):
            indexers_path.mkdir(parents=True)

        files = ((TEMPLATES / 'jackett' / 'Indexers').absolute()).glob('*')
        for file in files:
            shutil.copyfile(str(file.absolute()), indexers_path / file.name)

    def setup_nginx(self):
        config = self.path / 'Jackett' / 'ServerConfig.json'
        result = json.load(config.open('r'))
        result['BasePathOverride'] = '/trackers'
        json.dump(result, config.open('w'))



from pathlib import Path
import os
import shutil
import subprocess
import uuid

from .base import HomeServerApp
from .base import DOCKER_PATH
from .base import TEMPLATES


class DockerCompose(HomeServerApp):
    """Basic docker compose commands."""
    @property
    def env(self):
        return {'COMPOSE_PROJECT_NAME': self.meta['project_name']}

    def create_env_files(self):
        """Create environment files for docker copose"""
        root_pwd = uuid.uuid4().hex
        nextcloud_passwd = uuid.uuid4().hex
        (DOCKER_PATH / '.env_general').write_text(f"""PID={os.geteuid()}
    PUID={os.geteuid()}
    PGID={os.getegid()}""")
        (DOCKER_PATH /
         '.env_nextcloud').write_text(f"""NEXTCLOUD_ADMIN_USER=admin
    NEXTCLOUD_ADMIN_PASSWORD={nextcloud_passwd}
    NEXTCLOUD_UPDATE=1
    """)

    def copy_template(self):
        shutil.copyfile((TEMPLATES / 'docker-compose.yml').as_posix(),
                        (DOCKER_PATH / 'docker-compose.yml').as_posix())

        # Hack to allow nginx docker config mount
        nginx_path = DOCKER_PATH / 'data' / 'nginx'
        nginx_path.mkdir(exist_ok=True, parents=True)
        (nginx_path / 'nginx.conf').write_text('')

    def start(self):
        """Start."""
        print(f'starting {self.meta["project_name"]}')
        subprocess.check_output(
            ['docker-compose', '-p', self.meta['project_name'], 'up', '-d' ],
            cwd=DOCKER_PATH,
            env=self.env)

    def stop(self):
        """Stop."""
        subprocess.check_output(['docker-compose', 'stop'],
                                cwd=DOCKER_PATH,
                                env=self.env)

    def restart(self):
        self.stop()
        self.start()
